#!/usr/bin/perl -w

use POSIX qw(locale_h);

setlocale(LC_CTYPE, "pt_PT");

use locale;

use NAT::Client;
use CGI qw/:standard/;



my $corpora = NAT::Client::list_corpora();

my $crp = undef;
my $name;

if (param("crp")) {
  $crp = $corpora->{param("crp")}{id} || undef;
  $name = param("crp");
}
if (param("corpus")) {
  $crp = param("corpus");
  for (keys %$corpora) {
    $name = $_ if $corpora->{$_}{id} == $crp;
  }
}

($name) = keys %$corpora unless $name;


my $s = join("\n",
	     join("\n", map {
	       "source[\"$_\"]=\"$corpora->{$_}{source}\";"} keys %$corpora),
	     join("\n", map {
	       "target[\"$_\"]=\"$corpora->{$_}{target}\";"} keys %$corpora));

my $JSCRIPT = <<"EOS";

var source = new Array();
var target = new Array();

$s

function changeLanguages() {
  var corpus = document.getElementById('crp').value;
  document.getElementById('source').innerHTML = '('+source[corpus]+')';
  document.getElementById('target').innerHTML = '('+target[corpus]+')';
}

function go(l,c) {
  if (parseInt(navigator.appVersion)>=4)
    if (navigator.userAgent.indexOf("MSIE")>0) { //IE 4+
      var sel=document.selection.createRange();
      sel.expand("word");
      window.location="nat-dict.cgi?corpus=" + c + "&" + l + "=" + escape(sel.text)
    } else // NS4+
      window.location="nat-dict.cgi?corpus=" + c + "&" + l + "=" + escape(document.getSelection())
}

function help() {
   window.open('nat-search.cgi?HELP=1','NAT-QI Quick Help',
               'menubar=no,height=600,width=800,resizable=yes,toolbar=no,location=no,status=no');
}
EOS

print NAT::Client::my_header($JSCRIPT);

if (param("HELP")) {
  print NAT::Client::close_window();
  print help();
  print NAT::Client::my_footer();
  exit;
}



print div({-class=>"hlpbt",
           -onclick=>"help()"}, "Help  ");

print h1("NAT-QI: NATools Corpora Query Interface");

print start_form({-class=>"main"});
print "<table>";
print Tr(td({-rowspan=>'3'},submit("Search")),
	 td({-rowspan=>'3'}, "&nbsp;&nbsp;&nbsp;"),
	 td({-colspan=>6, -style=>"text-align: left"},
	    "Corpus: ",popup_menu(-onchange=>"changeLanguages();",
				  -name=>'crp',
				  -id => 'crp',
				  -default=>$name,
				  -values=>[keys %$corpora])));
print Tr(td(["Search on source language: ",
	     span({id=>"source"}, "($corpora->{$name}{source})"),
             textfield("l1"),
	     "&nbsp;&nbsp;&nbsp;&nbsp;",
	    ]),
	 td({-style=>"text-align: left"},label(checkbox(-name=>'sequence', -checked=>0,
							-value=>'ON', -label=>'Pattern Matching'))),
	 td(["&nbsp;&nbsp;&nbsp;&nbsp;",
	     "Result-set size",popup_menu(-name=>'count',
					  -values=>['20','50','100'])
	    ]));
print Tr(td(["Search on target language: ",
	     span({id=>"target"}, "($corpora->{$name}{target})"),
             textfield("l2"),
	     "&nbsp;&nbsp;&nbsp;&nbsp;"]),
	 td({-style=>"text-align: left"},
	    label(checkbox(-name=>'horiz', -checked=>0,
			    -value=>'ON', -label=>'Horizontal Mode')),
	   ));
print "</table>";
print end_form;


if ($crp && (param("l1") || param("l2"))) {
  print h1($name);
  print "<center>",a({-style=>"font-size: small;", -href=>"nat-about.cgi?corpus=$crp"},"about"),"</center>",br;

  my $results;

  $mod = (param("sequence") && param("sequence") eq "ON") ? "=" : "-";

  if (param("l1") && !param("l2")) {
    $results = NAT::Client::query({Count => param("count"),
				   Direction => "$mod>"},$crp, param("l1"));

  } elsif (param("l2") && !param("l1")) {
    $results = NAT::Client::query({Count => param("count"),
				   Direction => "<$mod"},$crp, param("l2"));

  } else {
    $results = NAT::Client::query({Count => param("count"),
				   Direction => "<$mod>"},$crp, param("l1"), param("l2"));
  }
  print "<table class='results' cellspacing='0'>";

  unless(param("horiz")) {
    print Tr(th(["#","%","Source Language","Target Language","Tools"]))
  } else {
    print Tr(th(["#","%","Source/Target Language","Tools"]))
  }

  my $i = 0;
  for (@$results) {
    $i++;

    # Code backwards compatibility O:-)
    $_->[4]=$_->[2]?sprintf("%.1f%%", 100*$_->[2]):"";
    $_->[2]=$_->[0];
    $_->[3]=$_->[1];

    # Highlight l1 if defined
    if (param("l1")) {
      my $x = param("l1");
      $x =~ s/^\s*(.*?)\s*$/$1/;
      if (param("sequence")) {
	$x =~ s/\*/\\S+/g;
	$_->[0] =~ s/\b$x\b/<span>$&<\/span>/gi;
      } else {
	$x =~ s/\*//g;
	for my $y (split /\s+/, $x) {
	  $_->[0] =~ s/\b$y\b/<span>$&<\/span>/gi;
	}
      }
    }

    # Highlight l2 if defined
    if (param("l2")) {
      my $x = param("l2");
      $x =~ s/^\s*(.*?)\s*$/$1/;
      if (param("sequence")) {
	$x =~ s/\*/\\S+/g;
	$_->[1] =~ s/\b$x\b/<span>$&<\/span>/gi;
      } else {
	$x =~ s/\*//g;
	for my $y (split /\s+/, $x) {
	  $_->[1] =~ s/\b$y\b/<span>$&<\/span>/gi;
	}
      }
    }

    my $FORM = start_form(-method => "POST",
			  -action => "nat-matrix.cgi").
			    hidden("corpus", $crp).
			      hidden("s1", $_->[2]).
				hidden("s2", $_->[3]).
				  submit("\\").end_form;

    if (param("horiz")) {
      print Tr(td({-class=>"entry", -rowspan=>2,
		   -style=>"border-bottom: solid 1px #999999; ".($i%2?"background-color: #ffffee":"")},$i),
	       td({-class=>"entry", -rowspan=>2,
		   -style=>"border-bottom: solid 1px #999999; ".($i%2?"background-color: #ffffee":"")},$_->[4]),
	       td({-class=>"entry",
		   -ondblclick=>"go('l1','$crp')",
		   -style=>"color: #333333; border-bottom: dotted 1px #999999; ".($i%2?"background-color: #ffffee":"")},
		  $_->[0]),
	       td({-class=>"entry", -rowspan=>2,
		   -style=>"border-bottom: solid 1px #999999; ".($i%2?"background-color: #ffffee":"")},
		  $FORM));
      print Tr(td({-class=>"entry",
		   -ondblclick=>"go('l2','$crp')",
		   -style=>"color: #333333; border-bottom: solid 1px #999999; ".($i%2?"background-color: #ffffee":"")},
		  $_->[1]));
    } else {
      print Tr(td({-class=>"entry",
		   -style=>"border-bottom: solid 1px #999999; ".($i%2?"background-color: #ffffee":"")},$i),
	       td({-class=>"entry",
		   -style=>"border-bottom: solid 1px #999999; ".($i%2?"background-color: #ffffee":"")},$_->[4]),
	       td({-class=>"entry",
		   -ondblclick=>"go('l1','$crp')",
		   -style=>"color: #333333; border-bottom: solid 1px #999999; ".($i%2?"background-color: #ffffee":"")}, $_->[0]),
	       td({-class=>"entry",
		   -ondblclick=>"go('l2','$crp')",
		   -style=>"color: #333333; border-bottom: solid 1px #999999; ".($i%2?"background-color: #ffffee":"")}, $_->[1]),
	       td({-class=>"entry",
		   -style=>"border-bottom: solid 1px #999999; ".($i%2?"background-color: #ffffee":"")},
		  $FORM));
      print "\n";
    }
  }
  print "</table>";
} else {
  print help();
}

print NAT::Client::my_footer();


sub linka {
  my ($txt,$l) = @_;
  # $txt =~ s!\w+!$& eq "span"?"span":"<a class=\"link\" href=\"nat-dict.cgi?$l=".lc($&)."\">$&</a>"!eg;
  return $txt;
}

sub help {
  return <<EOH;
  <div style="margin: 20px; border: solid 1px #000000; background-color: #ffffdd;">
     <h2 style="padding: 5px; border-bottom: solid 1px #000000; margin: 0px; background-color: #ddddbb">NAT-QI Help</h2>

     <div style="padding-left: 20px; padding-right: 20px">

     <p>NAT-QI (NATools Query Interface) is a web frontend to query and
     browse Parallel Corpora. For details about its architecture and
     associated tools see <a href="">this page</a>.</p>

     <p>This interface is querying a server (NATServer) with a
     specific parallel corpora, and a specific pair of
     languages. </p>

     <p><b>Toolbar Usage:</b></p>

     <dl>

      <dt><i>Simple Search:</i></dt>
      <dd>Enter a word in the <i>search on source language</i> or
     <i>search on target language</i> entries (or in both) to search
     for concordancies in the parallel corpora. Words are searched
     with no specific order.<br/>
      Examples:
      <a href="nat-search.cgi?crp=EuroParl&l1=europa&count=20">europa</a>,
      <a href="nat-search.cgi?crp=EuroParl&l2=dog%20bite&count=20">dog bite</a>,
      <a href="nat-search.cgi?crp=EuroParl&l1=europa&l2=dog&count=20">europa | dog</a> 
     </dd>

     <dt><i>Pattern Search:</i></dt>
     <dd>If you want to search for a specific
     sequence of words, click the <i>Pattern Matching</i> radio
     button. This option applies to both source and target language
     entries. It also has the feature of searching for patterns: enter
     some '*' in places you want any, not specific, word.<br/>
      Examples:
      <a href="nat-search.cgi?crp=EuroParl&sequence=ON&l1=parlamento%20europeu&count=20">parlamento europeu</a>,
      <a href="nat-search.cgi?crp=EuroParl&sequence=ON&l2=the%20*%20and%20the&count=20">the * and the</a>,
      <a href="nat-search.cgi?crp=EuroParl&sequence=ON&l1=europa&l2=european%20parliament&count=20">europa | european parliament</a>

     </dd>

     <dt><i>Horizontal Mode:</i></dt>
     <dd>The <i>Horizontal Mode</i> radio button let you change the
     layout of the output. Instead of two columns, one for each
     language, you get two lines, one for each language.</dd>

     <dt><i>Result-set Size:</i></dt>

     <dd> The <i>Result-set size</i> combo-box let you specify how many results
     you want.</dd>

     </dl>


     <p><b>Output Description:</b></p>

     <p>The standard output is a five column table: </p>
     <ol>
       <li>number which corresponds to the order in the result-set. Not
     very useful.</li>
       <li>a quality measure of the sentence pair.</li>
       <li>sentence in the source language.</li>
       <li>sentence in the target language.</li>
       <li>links to tools you can apply to the sentence pair.</li>
     </ol>

     <p><b>Related Tools Integration:</b></p>

     <ul>
     <li>
     Double-click any word in the source or target language to
     access its probabilistic dictionary entry. This tool will let you
     navigate through the dictionary.
     </li>
     <li>Use the <tt>[/]</tt> button in the tools column to access a
     sub-segment aligner, and sentence generalization tool.
     </li>
     </ul>

    </div>

  </div>
EOH
}
